#include <stdio.h>
#include <malloc.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

extern int isblank(int c);


#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))

#define CHAR_SPACE	' '
#define CHAR_TAB	'\t'
#define iscr(c)	(((c) == '\n') || ((c) == '\r'))

int no_tabs = 0;
int spaces_per_tab = 8;


void DoStrip(char *fi, char *fo)
{
	FILE *fip, *fop;
	struct stat stat_buf;
	off_t fi_non_blank_last, fo_non_blank_last;
	int c, blank_cnt, cur_line;


	if(!stat(fo, &stat_buf))
	{
	    fprintf(stderr,
		"%s: Already exists.\n",
		fo
	    );
	    return;
	}


	fip = fopen(fi, "r");
	if(fip == NULL)
	{
            fprintf(stderr,
                "%s: Cannot open.\n",
		fi
            );
	    return;
	}

        fop = fopen(fo, "w");
        if(fop == NULL)
        {
            fprintf(stderr,
                "%s: Cannot open.\n",
                fo
            );
	    fclose(fip);
            return;
        }


        fi_non_blank_last = 0;
        fo_non_blank_last = 0;
	cur_line = 0; 
	do
	{
	    c = fgetc(fip);
	    if(c == EOF)
		break;

	    if(iscr(c))
		cur_line++;

	    if(isblank(c))
	    {
		fi_non_blank_last = MAX(ftell(fip) - 1, 0);
		fo_non_blank_last = MAX(ftell(fop) - 1, 0);
		blank_cnt = ((c == CHAR_TAB) ? spaces_per_tab : 1);

		/* Seek to next non blank char. */
		do
		{
		    c = fgetc(fip);
		    if(c == EOF)
			break;

		    if(!isblank(c))
		    {
			if(iscr(c))
			{
			    fputc(c, fop);
			    cur_line++;
			}
			else
			{
			    /* Draw back blanks. */

			    if(no_tabs)
			    {
				int n;

                                for(n = 0; n < blank_cnt; n++)
                                    fputc(CHAR_SPACE, fop);
			    }
			    else
			    {
                                int n, nspaces, ntabs;

                                ntabs = blank_cnt / spaces_per_tab;
                                nspaces = blank_cnt - (ntabs * spaces_per_tab);

				for(n = 0; n < ntabs; n++)
				    fputc(CHAR_TAB, fop);

                                for(n = 0; n < nspaces; n++)
                                    fputc(CHAR_SPACE, fop);
			    }

			    fputc(c, fop);
			}
			break;
		    }
		    else
		    {
			blank_cnt += ((c == CHAR_TAB) ? spaces_per_tab : 1);
		    }

		} while(1);
	    }
	    else
	    {
		fputc(c, fop);
	    }

	} while(1);

	fclose(fip);
	fclose(fop);

	return;
}

int main(int argc, char **argv)
{

	if(argc < 3)
	{
	    fprintf(stderr,
"Usage: %s <infile> <outfile> [options]\n\
\n\
    Where [options] can be any of the following:\n\
\n\
        -t <n>                  Sepecifies tabs to be <n> characters\n\
                                (default is 8).\n\
        -a                      Replace all tabs with spaces.\n\
\n",
		argv[0]
	    );
	    return(1);
	}

	if(argc > 3)
	{
	    int i;
	    char *strptr;

	    for(i = 3; i < argc; i++)
	    {
		strptr = argv[i];
		if(strptr == NULL)
		    continue;

		if(!strcasecmp(strptr, "-a"))
		{
		    no_tabs = 1;
		}
		else if(!strcasecmp(strptr, "-t"))
		{
		    i++;
		    if(i < argc)
		    {
			spaces_per_tab = MAX(atoi(argv[i]), 1);
		    }
		    else
		    {
			fprintf(stderr,
			    "%s: Requires argument.\n",
			    argv[i - 1]
			);
		    }
		}

	    }
	}

	DoStrip(argv[1], argv[2]);

	return(0);
}
